@@ -1,6 +1,7 @@
 """
 Main.py
 """
+import argparse
 import asyncio
 import json
 import os
@@ -10,13 +11,15 @@ import uuid
 import requests
 import websockets.client as websockets
 
+DELIMITER = "\x1e"
+
 
 def append_identifier(msg: dict) -> str:
     """
     Appends special character to end of message to identify end of message
     """
     # Convert dict to json string
-    return json.dumps(msg) + ""
+    return json.dumps(msg) + DELIMITER
 
 
 class ChatHubRequest:
@@ -93,7 +96,7 @@ class Conversation:
         # Build request
         headers = {
             "accept": "application/json",
-            "accept-encoding": "gzip, deflate, br",
+            "accept-encoding": "gzip, deflate",
             "accept-language": "en-US,en;q=0.9",
             "content-type": "application/json",
             "sec-ch-ua": '"Microsoft Edge";v="111", "Not(A:Brand";v="8", "Chromium";v="111"',
@@ -113,12 +116,17 @@ class Conversation:
             "x-ms-useragent": "azsdk-js-api-client-factory/1.0.0-beta.1 core-rest-pipeline/1.10.0 OS/Linuxx86_64",
         }
         # Create cookies
-        cookies = {
-            "_U": os.environ.get("BING_U") or sys.argv[1],
-        }
+        if os.environ.get("BING_U") is None:
+            url = "https://bing.kpham.workers.dev/"
+            cookies = {}
+        else:
+            cookies = {
+                "_U": os.environ.get("BING_U"),
+            }
+            url = "https://www.bing.com/turing/conversation/create"
         # Send GET request
         response = requests.get(
-            "https://www.bing.com/turing/conversation/create",
+            url,
             headers=headers,
             cookies=cookies,
             timeout=30,
@@ -128,10 +136,10 @@ class Conversation:
         # Return response
         try:
             self.struct = response.json()
-        except json.decoder.JSONDecodeError:
+        except json.decoder.JSONDecodeError as exc:
             raise Exception(
-                "Authentication failed. You have not been accepted into the beta."
-            ) from json.decoder.JSONDecodeError
+                "Authentication failed. You have not been accepted into the beta.",
+            ) from exc
 
 
 class ChatHub:
@@ -150,7 +158,7 @@ class ChatHub:
             conversation_id=conversation.struct["conversationId"],
         )
 
-    async def ask(self, prompt: str) -> dict:
+    async def ask_stream(self, prompt: str) -> str:
         """
         Ask a question to the bot
         """
@@ -158,7 +166,7 @@ class ChatHub:
         if self.wss:
             if self.wss.closed:
                 self.wss = await websockets.connect(
-                    "wss://sydney.bing.com/sydney/ChatHub"
+                    "wss://sydney.bing.com/sydney/ChatHub",
                 )
                 await self.__initial_handshake()
         else:
@@ -168,14 +176,20 @@ class ChatHub:
         self.request.update(prompt=prompt)
         # Send request
         await self.wss.send(append_identifier(self.request.struct))
-        while True:
-            objects = str(await self.wss.recv()).split("")
+        final = False
+        while not final:
+            objects = str(await self.wss.recv()).split(DELIMITER)
             for obj in objects:
                 if obj is None or obj == "":
                     continue
                 response = json.loads(obj)
-                if response.get("type") == 2:
-                    return response
+                if response.get("type") == 1:
+                    yield False, response["arguments"][0]["messages"][0][
+                        "adaptiveCards"
+                    ][0]["body"][0]["text"]
+                elif response.get("type") == 2:
+                    final = True
+                    yield True, response
 
     async def __initial_handshake(self):
         await self.wss.send(append_identifier({"protocol": "json", "version": 1}))
@@ -189,6 +203,7 @@ class ChatHub:
             if not self.wss.closed:
                 await self.wss.close()
 
+
 class Chatbot:
     """
     Combines everything to make it seamless
@@ -201,7 +216,16 @@ class Chatbot:
         """
         Ask a question to the bot
         """
-        return await self.chat_hub.ask(prompt=prompt)
+        async for final, response in self.chat_hub.ask_stream(prompt=prompt):
+            if final:
+                return response
+
+    async def ask_stream(self, prompt: str) -> str:
+        """
+        Ask a question to the bot
+        """
+        async for response in self.chat_hub.ask_stream(prompt=prompt):
+            yield response
 
     async def close(self):
         """
@@ -264,11 +288,21 @@ async def main():
             await bot.reset()
             continue
         print("Bot:")
-        print(
-            (await bot.ask(prompt=prompt))["item"]["messages"][1]["adaptiveCards"][0][
-                "body"
-            ][0]["text"],
-        )
+        if args.no_stream:
+            print(
+                (await bot.ask(prompt=prompt))["item"]["messages"][1]["adaptiveCards"][
+                    0
+                ]["body"][0]["text"],
+            )
+        else:
+            wrote = 0
+            async for final, response in bot.ask_stream(prompt=prompt):
+                if not final:
+                    print(response[wrote:], end="")
+                    wrote = len(response)
+                    sys.stdout.flush()
+            print()
+        sys.stdout.flush()
     await bot.close()
 
 
@@ -285,4 +319,10 @@ if __name__ == "__main__":
         Enter twice to send message
     """,
     )
+    parser = argparse.ArgumentParser()
+    parser.add_argument("--no-stream", action="store_true")
+    parser.add_argument("--bing-cookie", type=str, default="")
+    args = parser.parse_args()
+    if args.bing_cookie:
+        os.environ["BING_U"] = args.bing_cookie
     asyncio.run(main())
